@@ -1,5 +1,6 @@
 library(longmemo)
 
+options(digits = 5) # since we want to "R CMD Rdiff" the output
 data(NileMin)
 fF <- FEXPest(NileMin, order.poly=3, pvalmax= .5, verbose=TRUE)
 fF
@@ -15,3 +16,5 @@ for(max.poly in 0:3) {
 
 fv
 plot(fv, type = "o", cex = 0.5)
+
+cat('Time elapsed: ', proc.time(),'\n') # for ``statistical reasons''
